package ru.amse.pomelov.starbugs.client.gui.frames;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import ru.amse.pomelov.starbugs.remote.GameServer;
import ru.amse.pomelov.starbugs.remote.MainServer;
import ru.amse.pomelov.starbugs.util.GameProperties;

public class GameCreationDialog extends JDialog {
	private static final long serialVersionUID = 1L;

	private class FinishAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public FinishAction() {
			putValue(AbstractAction.NAME, "Finish");
			putValue(AbstractAction.DEFAULT, true);
		}
		
		public void actionPerformed(ActionEvent e) {
			int width = 0;
			int height = 0;
			int num = 0;
			try {
				width = Integer.parseInt(detailsFieldWidth.getText());
				height = Integer.parseInt(detailsFieldWidth.getText());
				
				if ((width < 16) || (height < 16)) {
					JOptionPane.showMessageDialog(GameCreationDialog.this, "Field side should be at least 16");
					return;
				}
				
				num = Integer.parseInt(detailsPlayersNum.getText());
				if ((num < 2) || (num > 8)) {
					JOptionPane.showMessageDialog(GameCreationDialog.this, "Number of players should be over the range 2..8");
					return;
				}
			} catch (NumberFormatException e1) {
				JOptionPane.showMessageDialog(GameCreationDialog.this, "Incorrect number format!");
				return;
			}
			String gameName = detailsGameName.getText();
			GameProperties props = new GameProperties(width, height, num, gameName);
			
			try {
				MainServer server = (MainServer)Naming.lookup(serverURL.getText());
				String s = server.registerGame(props);
				if (s != null) {
					createdGameServer = ((GameServer) Naming.lookup(s));
					GameCreationDialog.this.dispose();
				} else {
					JOptionPane.showMessageDialog(GameCreationDialog.this, "Creation failed.\nTry to change game name.");
					return;
				}
			} catch (MalformedURLException e1) {
				JOptionPane.showMessageDialog(GameCreationDialog.this, "MalformedURLException");
			} catch (RemoteException e1) {
				JOptionPane.showMessageDialog(GameCreationDialog.this, "RemoteException");
			} catch (NotBoundException e1) {
				JOptionPane.showMessageDialog(GameCreationDialog.this, "NotBoundException");
			}
		}
	}
	
	
	
	private class CancelAction extends AbstractAction {
		private static final long serialVersionUID = 1L;

		public CancelAction() {
			putValue(AbstractAction.NAME, "Cancel");
		}
		
		public void actionPerformed(ActionEvent e) {
			createdGameServer = null;
			GameCreationDialog.this.dispose();
		}
	}
	
	
	
	private JTextField detailsGameName = new JTextField("Game Name");
	private JTextField detailsFieldWidth = new JTextField("100");
	private JTextField detailsFieldHeight = new JTextField("100");
	private JTextField detailsPlayersNum = new JTextField("2");
	
	private JPanel createDetailsPanel() {
		JPanel details = new JPanel(new GridLayout(3, 2, 5, 5));
		details.add(new JLabel("Game name: ", JLabel.RIGHT));
		details.add(detailsGameName);
		details.add(new JLabel("Field size: ", JLabel.RIGHT));
		JPanel sizeFields = new JPanel();
		sizeFields.setLayout(new BoxLayout(sizeFields, BoxLayout.LINE_AXIS));
		sizeFields.add(detailsFieldWidth);
		sizeFields.add(new JLabel(" x "));
		sizeFields.add(detailsFieldHeight);
		
		details.add(sizeFields);
		details.add(new JLabel("Players: ", JLabel.RIGHT));
		details.add(detailsPlayersNum);

		JPanel result = new JPanel(new BorderLayout());
		result.add(details, BorderLayout.NORTH);
		result.add(createButtonsPanel(), BorderLayout.SOUTH);
		TitledBorder border = BorderFactory.createTitledBorder("New game");
		border.setTitleJustification(TitledBorder.CENTER);
		result.setBorder(border);
		return result;
	}
	
	private JPanel createButtonsPanel() {
		JPanel buttons = new JPanel(new GridLayout(1, 2, 5, 5));
		buttons.setBorder(new EmptyBorder(5, 5, 5, 5));
		buttons.add(new JButton(new FinishAction()));
		buttons.add(new JButton(new CancelAction()));
		
		JPanel result = new JPanel(new BorderLayout());
		result.add(buttons, BorderLayout.EAST);
		return result;
	}
	
	
	private JTextField serverURL = new JTextField("rmi://localhost/server");
	private JPanel createServerURLPanel() {
		JPanel result = new JPanel(new BorderLayout());
		result.add(new JLabel(" Server URL : "), BorderLayout.WEST);
		result.add(serverURL, BorderLayout.CENTER);
		return result;
	}
	
	public GameCreationDialog(JFrame owner) {
		super(owner, "Create new game", true);
		setSize(320, 240);
		setLocation(360, 220);
		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		JPanel panel = new JPanel(new BorderLayout());
		getContentPane().add(panel);
		panel.add(createServerURLPanel(), BorderLayout.NORTH);
		panel.add(createDetailsPanel(), BorderLayout.CENTER);
		
		panel.getInputMap().put(KeyStroke.getKeyStroke((char)27), "cancel");
		panel.getActionMap().put("cancel", new CancelAction());
		
		setVisible(true);
	}	
	
	private static GameServer createdGameServer = null;
	
	public static GameServer createGame(JFrame owner) {
		new GameCreationDialog(owner);
		return createdGameServer;
	}
	
}
