package ru.amse.pomelov.starbugs.client.gui.frames;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.rmi.RemoteException;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import exceptions.ClientException;

import ru.amse.pomelov.starbugs.client.ClientDispatcher;
import ru.amse.pomelov.starbugs.remote.GameServer;
import ru.amse.pomelov.starbugs.remote.RemoteAdapter;

public class MenuFrame extends JFrame {
	private static final long serialVersionUID = 1L;
	
	private class EndOfGameListener extends RemoteAdapter {
		@Override
		public void endOfGame(int winner) throws RemoteException {
			createAction.setEnabled(true);
			joinAction.setEnabled(true);
			MenuFrame.this.setVisible(true);
		}
	}
	
	private GameServer newServer() {
		return GameCreationDialog.createGame(this);
	}
	
	private GameServer selectServer() {
		return GameSelectionDialog.selectGame(this);
	}
	
	private void createDispatcher(GameServer gameServer, boolean monitorOnly) {
		try {
			if (gameServer == null) {
				return;
			}
			new ClientDispatcher(gameServer, monitorOnly, new EndOfGameListener());
			if (!monitorOnly) {
				createAction.setEnabled(false);
				joinAction.setEnabled(false);
				MenuFrame.this.setVisible(false);
			}
		} catch (RemoteException e1) {
			JOptionPane.showMessageDialog(MenuFrame.this, "Failed to create client dispatcher.");
		} catch (ClientException e1) {
			e1.showMessage(MenuFrame.this);
		}		
	}
	
	private class CreateAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		private CreateAction() {
			putValue(NAME, "Create game");
		}
		
		public void actionPerformed(ActionEvent e) {
			GameServer gameServer = MenuFrame.this.newServer();
			createDispatcher(gameServer, false);
		}
	}
	
	private class JoinAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public JoinAction() {
			putValue(NAME, "Join game");
		}
		
		public void actionPerformed(ActionEvent e) {
			GameServer gameServer = MenuFrame.this.selectServer();
			createDispatcher(gameServer, false);
		}
	}
	/*TODO
	private class MonitorAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public MonitorAction() {
			putValue(NAME, "Monitor game");
		}
		
		public void actionPerformed(ActionEvent e) {
			GameServer gameServer = MenuFrame.this.selectServer();
			createDispatcher(gameServer, true);
		}
	}
	*/
	private class ExitAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
		
		public ExitAction() {
			putValue(NAME, "Quit");
		}
		
		public void actionPerformed(ActionEvent e) {
			System.exit(0);
		}
	}
	
	private final CreateAction createAction = new CreateAction();
	private final JoinAction joinAction = new JoinAction();
//	private final MonitorAction monitorAction = new MonitorAction();
	private final ExitAction exitAction = new ExitAction();
	

	
	public MenuFrame() {
		super("JSB");
		setSize(240, 200);
		setLocation(200, 200);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		JPanel panel = new JPanel(new GridLayout(3, 1, 5, 5));
		panel.setBorder(new TitledBorder("Main menu"));
		
		getContentPane().add(panel);
		
		panel.add(new JButton(createAction));
		panel.add(new JButton(joinAction));
//		panel.add(new JButton(monitorAction));
		panel.add(new JButton(exitAction));
		
		
		setVisible(true);
		
	}
	
	public static void main(String[] args) {
		new MenuFrame();
	}
}




