package ru.amse.pomelov.starbugs.turns;

import java.util.EnumSet;
import java.util.LinkedList;
import java.util.List;

import ru.amse.pomelov.starbugs.server.GameDispatcher;
import ru.amse.pomelov.starbugs.units.AbstractUnit;
import ru.amse.pomelov.starbugs.units.AbstractUnitTypes;
import ru.amse.pomelov.starbugs.units.Unit;
import ru.amse.pomelov.starbugs.units.Units;
import ru.amse.pomelov.starbugs.util.FieldPosition;

public class AttackTurn extends AbstractTurn {
	
	private static final long serialVersionUID = 1L;

	public AttackTurn(FieldPosition object, FieldPosition subject) {
		super(object, subject);
	}

	@Override
	public int acceptTurn(GameDispatcher gd) {
		if (!gd.isActive(player, myObject)) {
			return -1;
		}
		if (player != gd.cellOwner(myObject)) {
			return -1;
		}
		if (gd.isEmptyCell(mySubject)) {
			return -1;
		}
		if (player == gd.cellOwner(mySubject)) {
			return -1;
		}
		
		AbstractUnit objectUnit = gd.getUnit(myObject);
		int radius = GameDispatcher.distanse(myObject, mySubject);
		int allowedRadiuses = objectUnit.getAttackRadiuses();
		if ((allowedRadiuses & radius) == 0) {
			return -1;
		}
		
		AbstractUnitTypes subjectType = gd.getUnit(mySubject).getType();
		EnumSet<AbstractUnitTypes> allowedTypes = objectUnit.getAttackTypes();
		if (!allowedTypes.contains(subjectType)) {
			return -1;
		}
		
		return objectUnit.getAttackCost();
	}

	@Override
	public void makeTurn(GameDispatcher gd) {
		List<AbstractUnit> unitList = new LinkedList<AbstractUnit>();
		unitList.add(new Unit(mySubject, player, Units.BLOCK));
		gd.putUnits(unitList);
	}

}
