package ru.amse.pomelov.starbugs.turns;

import java.util.LinkedList;
import java.util.List;

import ru.amse.pomelov.starbugs.config.UnitAttributes;
import ru.amse.pomelov.starbugs.server.GameDispatcher;
import ru.amse.pomelov.starbugs.units.AbstractUnit;
import ru.amse.pomelov.starbugs.units.Building;
import ru.amse.pomelov.starbugs.units.Edifice;
import ru.amse.pomelov.starbugs.units.Edifices;
import ru.amse.pomelov.starbugs.util.FieldPosition;
import ru.amse.pomelov.starbugs.util.MaskIterator;

public class BuildTurn extends AbstractTurn {

	private static final long serialVersionUID = 1L;
	
	private final Edifices myType;
	
	public BuildTurn(FieldPosition object, FieldPosition subject, Edifices type) {
		super(object, subject);
		myType = type;
	}

	@Override
	public int acceptTurn(GameDispatcher gd) {
		if (!gd.isActive(player, myObject)) {
			return -1;
		}
		if (player != gd.cellOwner(myObject)) {
			return -1;
		}
		if (!gd.getUnit(myObject).isAbleToBuild()) {
			return -1;
		}
		
		if (!gd.isConnectedToBuildings(player, myType.getBuildingType().getDependence(), myObject)) {
			return -1;
		}
		
		boolean connected = false;
		for (FieldPosition c : new MaskIterator(myType.getMask(), mySubject)) {
			try {
				if (!gd.isEmptyCell(c)) {
					return -1;
				}
				if (GameDispatcher.areNeighbours(myObject, c)) {
					connected = true;
				}
			} catch (IndexOutOfBoundsException e) {
				return -1;
			}
		}
		
		return connected ? UnitAttributes.BUILDING_START_COST : -1;
	}

	@Override
	public void makeTurn(GameDispatcher gd) {
		Edifice e = new Edifice(mySubject, myType);
		List<AbstractUnit> unitList = new LinkedList<AbstractUnit>();
		for (FieldPosition p : new MaskIterator(myType.getMask(), mySubject)) {
			AbstractUnit u = new Building(p, -1, myType.getBuildingType());
			unitList.add(u);
		}
		gd.putUnits(unitList);
		gd.addBuildingSite(mySubject, player, e);
	}

	
}
