package ru.amse.pomelov.starbugs.units;

import java.util.EnumSet;

import javax.swing.ImageIcon;

import ru.amse.pomelov.starbugs.client.gui.IconFactory;
import ru.amse.pomelov.starbugs.util.FieldPosition;


/**
 * The abstract class Unit represents the game unit. 
 * 
 * @author Alexei Pomelov
 *
 */
public class Unit extends AbstractUnit{

	private static final long serialVersionUID = 1L;
	
	private final Units myUnitType;
	
	public Unit(FieldPosition position, int player, Units type) {
		super(position, player);
		myUnitType = type;
	}

	@Override
	public int getCost() {
		return myUnitType.getCost();
	}
	
	@Override
	public int getAttackCost() {
		return myUnitType.getAttackCost();
	}
	
	@Override
	public int getAttackRadiuses() {
		return myUnitType.getAttackRadiuses();
	}
	
	@Override
	public EnumSet<AbstractUnitTypes> getAttackTypes() {
		return myUnitType.getAttackTypes();
	}
	
	@Override
	public boolean isAbleToBuild() {
		return myUnitType.isAbleToBuild();
	}
	
	public Units getUnitType() {
		return myUnitType;
	}
	
	@Override
	public AbstractUnitTypes getType() {
		return myUnitType.getUnitType();
	}

	@Override
	public String toString() {
		if (myUnitType == Units.ARCHER) {
			return " A ";
		} else if (myUnitType == Units.BALLISTA) {
			return " B ";
		} else if (myUnitType == Units.BLOCK) {
			return "BLK";
		} else if (myUnitType == Units.CATAPULT) {
			return " C ";
		} else if (myUnitType == Units.RUNNER) {
			return " R ";
		} else if (myUnitType == Units.SOLDIER) {
			return " S ";
		} else if (myUnitType == Units.TANK) {
			return " T ";
		} else if (myUnitType == Units.WORKER) {
			return " W ";
		} else {
			return "   ";
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Unit)) {
			return false;
		}
		Unit you = (Unit)obj;
		return super.equals(obj) && (myUnitType == you.getUnitType());
	}

	@Override
	public int hashCode() {
		return super.hashCode() + 47 * myUnitType.hashCode();
	}
	
	@Override
	public ImageIcon acceptIconFacory() {
		return IconFactory.createIcon(this);
	}
	
}
