package ru.amse.pomelov.starbugs.server;

import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ru.amse.pomelov.starbugs.units.AbstractUnit;
import ru.amse.pomelov.starbugs.units.Building;
import ru.amse.pomelov.starbugs.units.Buildings;
import ru.amse.pomelov.starbugs.units.Edifice;
import ru.amse.pomelov.starbugs.util.FieldPosition;
import ru.amse.pomelov.starbugs.util.MaskIterator;

public class PlayerAccount implements Serializable {
	private static final long serialVersionUID = 1L;

	private int myMaxTurnTime;

	private int myTurnTimeLeft;

	private final int myNumber;
	
	//private boolean lostFlag = false;
	
	public PlayerAccount(int maxTurnTime, int number) {
		myMaxTurnTime = maxTurnTime;
		myTurnTimeLeft = myMaxTurnTime;
		myNumber = number;
	}

	public int getTurnTimeLeft() {
		return myTurnTimeLeft;
	}

	public final Map<Buildings, LinkedList<Edifice>> myEdifices
			= new HashMap<Buildings, LinkedList<Edifice>>();

	public final Set<Edifice> myBuildingSites = new HashSet<Edifice>();

	public void endOfTurn(GameDispatcher gd) {
		for (Iterator<Edifice> iter = myBuildingSites.iterator(); iter.hasNext();) {
			Edifice e = iter.next();
			if (e.proceed()) {
				LinkedList<Edifice> list = myEdifices.get(e.getEdificeType().getBuildingType());
				if (list == null) {
					list = new LinkedList<Edifice>();
					myEdifices.put(e.getEdificeType().getBuildingType(), list);
				}
				list.add(e);
				List<AbstractUnit> unitList = new LinkedList<AbstractUnit>();
				for (FieldPosition p : new MaskIterator(e.getEdificeType().getMask(), e.getPosition())) {
					AbstractUnit u = new Building(p, myNumber, e.getEdificeType().getBuildingType());
					unitList.add(u);
				}
				gd.putUnits(unitList);
				iter.remove();
			}
		}
		myTurnTimeLeft = myMaxTurnTime;
	}
	
	public void updateEdifices(GameDispatcher gd) {
		for (LinkedList<Edifice> list : myEdifices.values()) {
			for (Iterator<Edifice> iter = list.iterator(); iter.hasNext(); ) {
				Edifice edifice = iter.next();
				boolean buildingIsAlive = false;
				boolean[][] mask = edifice.getEdificeType().getMask();
				FieldPosition strt = edifice.getPosition();
				for (FieldPosition p : new MaskIterator(mask, strt)) {
					AbstractUnit u = gd.getUnit(p);
					if (u == null) {
						throw new IllegalStateException();
					}
					if (u.getPlayer() == myNumber) {
						buildingIsAlive = true;
					}
				}
				if (!buildingIsAlive) {
					iter.remove();
				}
			}
		}
	}
	
	public void decreaseTime(int d) {
		myTurnTimeLeft -= d;
	}
}
