package ru.amse.pomelov.starbugs.server;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Collection;
import java.util.Scanner;

public class ServerStart {
	
	private static MainServerImpl server = null;
	private static String bind = "rmi://localhost/server";
	private static Scanner scanner = new Scanner(System.in);
	
	
	public static void main(String[] args) {
		
		System.out.println("StarBugs server v1.0");
		System.out.println("Type HELP to get brief info");
		
		
		
		String cmd = "";
		while (true) {
			System.out.print(">> ");
			cmd = scanner.nextLine();
			
			if ("help".equalsIgnoreCase(cmd)) {
				help();
			} else if ("getBind".equalsIgnoreCase(cmd)) {
				getBind();
			} else if ("setBind".equalsIgnoreCase(cmd)) {
				setBind();
			} else if ("kill".equalsIgnoreCase(cmd)) {
				kill();
			} else if ("create".equalsIgnoreCase(cmd)) {
				create();
			} else if (("exit".equalsIgnoreCase(cmd)) || ("quit".equalsIgnoreCase(cmd))) {
				if (exit()) {
					System.exit(0);
				}
			} else {
				System.out.println("Bad command name.");
			}
		}
	}


	private static void help() {
		System.out.println("Available commands:");
		System.out.println(" -- GETBIND");
		System.out.println(" -- SETBIND");
		System.out.println(" -- CREATE");
		System.out.println(" -- KILL");
		System.out.println(" -- ");
		System.out.println(" -- EXIT/QUIT");
	}
	
	private static void getBind() {
		System.out.println("Current bind name is: '" + bind + "'");
	}
	
	private static void setBind() {
		if (server != null) {
			System.out.println("You couldn't change server location when it's alive.");
		} else {
			
			System.out.println("Input new server location:");
			bind = scanner.nextLine();
			getBind();
		}
	}
	
	private static boolean kill() {
		if (server == null) {
			System.out.println("The server isn't alive.");
			return true;
		}
		
		try {
			Collection<String> games = server.viewGames();
			if (games.size() > 0) {
				System.out.println("Game list isn't empty. All games will be stoped.\n"
						+ "Proceed anymore?(yes/no)");
				if (!"yes".equalsIgnoreCase(scanner.nextLine())) {
					return false;
				}
			}
			server.forceKill();
			UnicastRemoteObject.unexportObject(server, false);
			server = null;
			return true;
		} catch (RemoteException e) {
		}
		return true;
	}
	
	private static void create() {
		if (server != null) {
			System.out.println("Server is already created.");
		} else {
			server = new MainServerImpl(bind);
			try {
				UnicastRemoteObject.exportObject(server);
				Naming.rebind(bind, server);
				System.out.println("Created server on '" + bind + "'");
			} catch (RemoteException e) {
				server = null;
				System.out.println("Failed to export server.");
			} catch (MalformedURLException e) {
				server = null;
				System.out.println("The bind URL is Malformed. Use SETBIND to change it.");
			}
		}
	}
	
	private static boolean exit() {
		if (server != null) {
			return kill();
		}
		return true;
	}
}
